//
//  ViewController.m
//  BTSP02
//
//  Created by 富樫 豊彦 on 2019/03/05.
//  Copyright © 2019 富樫 豊彦. All rights reserved.
//

#import "ViewController.h"
#import <CoreBluetooth/CoreBluetooth.h>

@interface ViewController () <CBCentralManagerDelegate, CBPeripheralDelegate>
@property (strong, nonatomic) CBCentralManager      *centralManager;
@property (strong, nonatomic) CBPeripheral          *discoveredPeripheral;
@property (strong, nonatomic) CBCharacteristic      *charMainVoluem;
@property (strong, nonatomic) CBCharacteristic      *charAVoluem;
@property (strong, nonatomic) CBCharacteristic      *charBVoluem;
@property (strong, nonatomic) CBCharacteristic      *charBtVoluem;
@property (strong, nonatomic) CBCharacteristic      *charABalance;
@property (strong, nonatomic) CBCharacteristic      *charBBalance;
@property (strong, nonatomic) CBCharacteristic      *charBtBalance;
@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    NSLog(@"BT Init");
    self.centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Bluetooth-LE central


- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    NSLog(@"BT update %ld", (long)central.state);
    if (central.state != CBCentralManagerStatePoweredOn) {
        return;
    }
    [self scan];
}


- (void)scan
{
    [self.centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:BTSP_SERVICE_UUID]] options:nil];
    NSLog(@"Scanning...");
}


- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    NSLog(@"Discovered %@ at %@db", peripheral.name, RSSI);
    [self.msg setText:[[NSString alloc] initWithFormat:@"%@ at %@[db]", peripheral.name, RSSI]];
    if (self.discoveredPeripheral != peripheral) {
        self.discoveredPeripheral = peripheral;
        if ((![[peripheral.name substringWithRange:NSMakeRange(0, 5)] compare:@"Audio"]) ||
            (![[peripheral.name substringWithRange:NSMakeRange(0, 4)] compare:@"BTSP"])) {
            NSLog(@"Connecting to peripheral %@", peripheral);
            [self.centralManager connectPeripheral:peripheral options:nil];
        } else {
            NSLog(@"Connect skip %@", peripheral);
        }
    }
}


- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Failed to connect to %@. (%@)", peripheral, [error localizedDescription]);
    [self cleanup];
}


- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"Peripheral Connected %@", peripheral);
    [self.msg setText:[[NSString alloc] initWithFormat:@"Connected: %@", peripheral.name]];
    [self.centralManager stopScan];
    peripheral.delegate = self;
    [peripheral discoverServices:@[[CBUUID UUIDWithString:BTSP_SERVICE_UUID]]];
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
{
    if (error) {
        NSLog(@"Error discovering services: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBService *service in peripheral.services) {
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID1]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID2]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID3]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID4]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID5]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID6]] forService:service];
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID7]] forService:service];
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error
{
    if (error) {
        NSLog(@"Error discovering characteristics: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBCharacteristic *c in service.characteristics) {
        NSLog(@"Char=%@", c);
        if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID1]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charMainVoluem = c;
        } else if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID2]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charAVoluem = c;
        } else if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID3]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charABalance = c;
        } else if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID4]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charBVoluem = c;
        } else if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID5]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charBBalance = c;
        } else if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID6]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charBtVoluem = c;
        } else if ([c.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID7]]) {
            [peripheral readValueForCharacteristic:c];
            [peripheral setNotifyValue:YES forCharacteristic:c];
            self.charBtBalance = c;
        }
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    int a;
    
    if (error) {
        NSLog(@"Error discovering characteristics: %@", [error localizedDescription]);
        return;
    }
    NSLog(@"Updated: %@", characteristic.UUID);
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID1]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.mainVolume setValue: a/100.0];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID2]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.aVolume setValue: a/100.0];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID3]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.aBalance setValue: a/100.0];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID4]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.bVolume setValue: a/100.0];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID5]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.bBalance setValue: a/100.0];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID6]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.btVolume setValue: a/100.0];
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BTSP_CHARACTERISTIC_UUID7]]) {
        a = *((short *)[characteristic.value bytes]);        // Byte order is Little endian
        [self.btBalance setValue: a/100.0];
    }
}


- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Peripheral Disconnected");
    self.discoveredPeripheral = nil;
    [self scan];
    [self.msg setText:@"Waiting a reconnect..."];
}

- (void)cleanup {
    NSLog(@"BT Cleanup");
    return;
}

- (IBAction)changeMainVolume:(id)sender{
    unsigned char   buff[2];
    int a;

    NSLog(@"changeMainVolume");
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charMainVoluem type:CBCharacteristicWriteWithResponse];
}

- (IBAction)changeAVolume:(id)sender{
    unsigned char   buff[2];
    int a;
    
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charAVoluem type:CBCharacteristicWriteWithResponse];
}

- (IBAction)changeABalance:(id)sender{
    unsigned char   buff[2];
    int a;
    
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charABalance type:CBCharacteristicWriteWithResponse];
}

- (IBAction)changeBVolume:(id)sender{
    unsigned char   buff[2];
    int a;
    
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charBVoluem type:CBCharacteristicWriteWithResponse];
}

- (IBAction)changeBBalance:(id)sender{
    unsigned char   buff[2];
    int a;
    
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charBBalance type:CBCharacteristicWriteWithResponse];
}

- (IBAction)changeBtVolume:(id)sender{
    unsigned char   buff[2];
    int a;
    
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charBtVoluem type:CBCharacteristicWriteWithResponse];
}

- (IBAction)changeBtBalance:(id)sender{
    unsigned char   buff[2];
    int a;
    
    a = self.mainVolume.value * 100;
    buff[0] = a;
    buff[1] = a >> 8;
    NSData *val = [[NSData alloc] initWithBytes:&buff length:sizeof(buff)];
    [self.discoveredPeripheral writeValue:val forCharacteristic:self.charBtBalance type:CBCharacteristicWriteWithResponse];
}


@end
